from abc import abstractmethod

import psycopg2
from django.contrib.gis.geos import GEOSException, MultiPolygon, Polygon, MultiPoint, MultiLineString
from django.core.files.uploadedfile import UploadedFile

from konova.models import Geometry


class BaseMigrater:
    options = None
    db_connection = None

    def __init__(self, options: dict):
        self.options = options
        self.connect_db()

    def __del__(self):
        if self.db_connection is not None:
            self.db_connection.close()

    def connect_db(self):
        if self.options is None:
            return
        conn = psycopg2.connect(
            host=self.options["host"],
            database="ksp",
            user=self.options["db_user"],
            password=self.options["db_pw"],
        )
        print("Connected to ksp db...")
        self.db_connection = conn

    @abstractmethod
    def migrate(self):
        raise NotImplementedError("Must be implemented in subclass")

    def _migrate_geometry(self, instance, db_result: tuple):
        try:
            # db_result[2] == polygon, db_result[3] == line, db_result[4] == point
            if db_result[2] is not None:
                db_result_geom = MultiPolygon.from_ewkt(db_result[2])
            elif db_result[3] is not None:
                db_result_geom = MultiLineString.from_ewkt(db_result[3])
                db_result_geom = db_result_geom.buffer(0.00001, 1)
                if isinstance(db_result_geom, Polygon):
                    db_result_geom = MultiPolygon(db_result_geom)
            elif db_result[4] is not None:
                db_result_geom = MultiPoint.from_ewkt(db_result[4])
                db_result_geom = db_result_geom.buffer(0.00001, 1)
                if isinstance(db_result_geom, Polygon):
                    db_result_geom = MultiPolygon(db_result_geom)
            else:
                db_result_geom = None
        except GEOSException:
            print(f"Malicious geometry on {db_result}")
            return
        geom = instance.geometry or Geometry()
        geom.geom = db_result_geom
        geom.save()
        # celery_update_parcels.delay(geom.id)

        instance.geometry = geom
        return instance

    def _migrate_documents(self, instance, document_cls, db_result):
        db_result_identifier = f"'{db_result[0]}'"
        tmp_cursor = self.db_connection.cursor()
        tmp_cursor.execute(
            'select '
                'doc.pfad, '
                'doc."Bemerkung", '
                'doc."Datum" '
            'from "OBJ_MASTER" om '
                'left join "LINFOS" linf on om."GISPADID"=linf."GISPADID" '
                'left join "Objektphotos" doc on linf."PKEY"=doc."FKEY" '
            'where '
                f'om."KENNUNG"={db_result_identifier} and '
                'doc.pfad is not null '
        )
        doc_results = tmp_cursor.fetchall()
        if len(doc_results) > 0:
            for doc_result in doc_results:
                doc_path = doc_result[0]
                doc_comment = doc_result[1]
                doc_date = doc_result[2]
                try:
                    with open(doc_path, encoding="latin1") as file:
                        file = UploadedFile(file)
                        doc_title = "Migrierte Datei"
                        doc_date = doc_date or "1970-01-01"
                        doc_exists = document_cls.objects.filter(
                            instance=instance,
                            title=doc_title,
                            comment=doc_comment,
                            date_of_creation=doc_date
                        ).exists()
                        if doc_exists:
                            continue
                        doc = document_cls.objects.create(
                            title=doc_title,
                            comment=doc_comment,
                            file=file,
                            date_of_creation=doc_date,
                            instance=instance,
                        )
                except FileNotFoundError:
                    print(f"------ !!! File not found: {doc_path}")
        tmp_cursor.close()
        return instance